#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <curses.h>
#include <math.h>
#include <time.h>

#define  CCR_types   6
#define  range_types 5
#define  node_types  5
#define  outdegree_types 6
#define  shape_types  3
#define  numsample   25
#define FILE_NAME       "TEST_FILE"

void  delete_old_copies();
void  analyze_results();
void  update_global_results();

FILE  *out, *out2, *finals_m1, *finals_m2, *finals_m3, 
  *finals_m4, *finals_m5;

int    numnode,            /* graph node information */
       out_degree,         /* average outdegree */
       numtpe,             /* number of processors */
       num_item;

float  CCR, range, shape;
float  CCR_array[CCR_types] = {0.1, 0.5, 1.0, 2.0, 5.0, 10.0}; 
float  range_array[range_types] = {0.1, 0.25, 0.5, 0.75, 1.0};
float  shape_array[shape_types] = {0.5, 1.0, 2.0};
int    node_array[node_types] = {20, 40, 60, 80, 100};
int    outdegree_array[outdegree_types] = {1, 2, 3, 4, 5, 100};
char   dag_info[30];
float  g_slr_m1, g_slr_m2, g_slr_m3, g_slr_m4, g_slr_m5, 
       g_time_m1, g_time_m2, g_time_m3, g_time_m4, g_time_m5;
int    g_item;

void main(int argc, char **argv)
{
  char  g[50], p[50], graph_type, temp_c;
  int  CCR_pos, node_pos, range_pos, outdegree_pos, shape_pos,
       k, i, m, pe_used, pe;
  long tnum;

  g_slr_m1 = g_time_m1 = 0.0;
  g_slr_m2 = g_time_m2 = 0.0;
  g_slr_m3 = g_time_m3 = 0.0;
  g_slr_m4 = g_time_m4 = 0.0;
  g_slr_m5 = g_time_m5 = 0.0;
  g_item = 0;

  if (argc < 2) {
    fprintf(stderr, "usage: %s numprocessor\n", argv[0]);
    exit(0);
  }
  sscanf(argv[1],"%d",&numtpe);
  if ((out = fopen("node_ccr.exp", "a")) == NULL) {
    fprintf(stderr, "Error in appending the file %s", "EXP_TYPE");
    exit(0);
  }
  if ((out2 = fopen("node_ccr_avg.exp", "a")) == NULL) {
    fprintf(stderr, "Error in appending the file %s", "EXP_TYPE");
    exit(0);
  }

  for (node_pos = 0; node_pos<node_types; node_pos++) {
     numnode = node_array[node_pos];
     fprintf(out2,"%d ", numnode);
     for (CCR_pos = 0; CCR_pos<CCR_types; CCR_pos++) {
        CCR = CCR_array[CCR_pos];
        num_item = 0;
	g_item++;
	sprintf(dag_info, "%d  %5.2f", numnode, CCR); 
	fprintf(out,"%s ", dag_info);
        for (shape_pos = 0; shape_pos<shape_types; shape_pos++) {
	   shape = shape_array[shape_pos];
	   for (outdegree_pos=0; outdegree_pos<outdegree_types; outdegree_pos++) {
	      out_degree = outdegree_array[outdegree_pos];
	      for (range_pos=0; range_pos<range_types; range_pos++) {
		range = range_array[range_pos];
		sprintf(g,"gen %d %5.2f %3.1f %d %4.2f %d", numnode, CCR, shape, 
			out_degree, range, numtpe);
		/* execute numsamples execution for each case */
		for (k=1; k<=numsample; k++) {
		  num_item++;
		  tnum = (int) ceil(10000000.0 *(double) rand() 
				    / (double) RAND_MAX); 
		  sprintf(p, "%s %ld\n", g, tnum);
		  printf("\n%s\n", p);
		  system(p);   /* graph  generated  */
		  /* printf("done %s\n", p); */
		  sprintf(p,"m1 %d",numtpe);  
		  system(p);   
		  sprintf(p,"m2 %d",numtpe);  
		  system(p);   
		  sprintf(p,"m3 %d",numtpe);  
		  system(p);   
		  sprintf(p,"m44 %d",numtpe);  
		  system(p);   
		  sprintf(p,"m5 %d",numtpe);  
		  system(p);   
		}
	      }
	   }
	}
	printf("%d\n", num_item);
	analyze_results();
	delete_old_copies(); 
     }
     update_global_results();
  }
  fclose(out);  fclose(out2);
}



void  analyze_results() {
  int  item_m1, item_m2, item_m3, item_m4, item_m5, num_pe_used;
  float slr_m1, slr_m2, slr_m3, slr_m4, slr_m5,
    time_m1, time_m2, time_m3, time_m4, time_m5,
    diff_Time, sum_min, sum_max, min_cp_nc, min_cp, sch_length;
  
  
  if ((finals_m1 = fopen("m1.mean", "r")) == NULL) {
    fprintf(stderr, "Error in reading the file");
    exit(0);
  }

  if ((finals_m2 = fopen("m2.mean", "r")) == NULL) {
    fprintf(stderr, "Error in reading the file");
    exit(0);
  }
  if ((finals_m3 = fopen("m3.mean", "r")) == NULL) {
    fprintf(stderr, "Error in reading the file");
    exit(0);
  }
  if ((finals_m4 = fopen("m4.mean", "r")) == NULL) {
    fprintf(stderr, "Error in reading the file");
    exit(0);
  }
  if ((finals_m5 = fopen("m5.mean", "r")) == NULL) {
    fprintf(stderr, "Error in reading the file");
    exit(0);
  }

  item_m1 = item_m2 = item_m3 = item_m4 = item_m5 = 0;
  slr_m1 = time_m1 = 0.0;
  slr_m2 = time_m2 = 0.0;
  slr_m3 = time_m3 = 0.0;
  slr_m4 = time_m4 = 0.0;
  slr_m5 = time_m5 = 0.0;

  while (!feof(finals_m1)) {
    /** M1.c **/
    fscanf(finals_m1, "%f  %f  %f  %f  %f  %f  %d", &sum_min, &sum_max, 
	   &min_cp_nc, &min_cp, &sch_length, &diff_Time, &num_pe_used);
    if  (!feof(finals_m1)) {
      slr_m1 += sch_length / min_cp_nc;        
      time_m1 += diff_Time;
      item_m1++;
    }
  }

  while (!feof(finals_m2)) {
    /** M2.c **/
    fscanf(finals_m2, "%f  %f  %f  %f  %f  %f  %d", &sum_min, &sum_max, 
	   &min_cp_nc, &min_cp, &sch_length, &diff_Time, &num_pe_used);
    if  (!feof(finals_m2)) {
      slr_m2 += sch_length / min_cp_nc;        
      time_m2 += diff_Time;
      item_m2++;
    }
  }

  while (!feof(finals_m3)) {
    /** M3.c **/
    fscanf(finals_m3, "%f  %f  %f  %f  %f  %f  %d", &sum_min, &sum_max, 
	   &min_cp_nc, &min_cp, &sch_length, &diff_Time, &num_pe_used);
    if  (!feof(finals_m3)) {
      slr_m3 += sch_length / min_cp_nc;        
      time_m3 += diff_Time;
      item_m3++;
    }
  }

  while (!feof(finals_m4)) {
    /** M4.c **/
    fscanf(finals_m4, "%f  %f  %f  %f  %f  %f  %d", &sum_min, &sum_max, 
	   &min_cp_nc, &min_cp, &sch_length, &diff_Time, &num_pe_used);
    if  (!feof(finals_m4)) {
      slr_m4 += sch_length / min_cp_nc;        
      time_m4 += diff_Time;
      item_m4++;
    }
  }

  while (!feof(finals_m5)) {
    /** M1.c **/
    fscanf(finals_m5, "%f  %f  %f  %f  %f  %f  %d", &sum_min, &sum_max, 
	   &min_cp_nc, &min_cp, &sch_length, &diff_Time, &num_pe_used);
    if  (!feof(finals_m5)) {
      slr_m5 += sch_length / min_cp_nc;        
      time_m5 += diff_Time;
      item_m5++;
    }
  }

  fclose(finals_m1);
  fclose(finals_m2);
  fclose(finals_m3);
  fclose(finals_m4);
  fclose(finals_m5);
   
  /* slr_h, slr_c, slr_m, slr_d, slr_l, speedup, ....*/
  fprintf(out,"%f %f %f %f %f %f %f %f %f %f\n", 
	  slr_m1/item_m1, slr_m2/item_m2, slr_m3/item_m3, 
	  slr_m4/item_m4, slr_m5/item_m5,
	  time_m1/item_m1, time_m2/item_m2, time_m3/item_m3,
	  time_m4/item_m4, time_m5/item_m5);
  
  g_slr_m1 += slr_m1/item_m1;
  g_time_m1 += time_m1/item_m1;
  g_slr_m2 += slr_m2/item_m2;
  g_time_m2 += time_m2/item_m2;
  g_slr_m3 += slr_m3/item_m3;
  g_time_m3 += time_m3/item_m3;
  g_slr_m4 += slr_m4/item_m4;
  g_time_m4 += time_m4/item_m4;
  g_slr_m5 += slr_m5/item_m5;
  g_time_m5 += time_m5/item_m5;
}

void update_global_results() {
  fprintf(out2,"%f %f %f %f %f %f %f %f %f %f\n", 
	  g_slr_m1/g_item, g_slr_m2/g_item,  g_slr_m3/g_item, 
	  g_slr_m4/g_item, g_slr_m5/g_item,
	  g_time_m1/g_item, g_time_m2/g_item, g_time_m3/g_item,
	  g_time_m4/g_item, g_time_m5/g_item);
  
  g_slr_m1 = g_time_m1 = 0.0;
  g_slr_m2 = g_time_m2 = 0.0;
  g_slr_m3 = g_time_m3 = 0.0;
  g_slr_m4 = g_time_m4 = 0.0;
  g_slr_m5 = g_time_m5 = 0.0;
  g_item = 0;
}

void  delete_old_copies() {
  /* system("rm *.out"); */
  system("rm *.mean");
  system("rm *.median"); 
}






